# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_string import AdditionalDataString
from .attachments import Attachments
from .bill_data_payment_terms import BillDataPaymentTerms
from .bill_item import BillItem
from .datenullable import Datenullable
from .discount import Discount
from .duty_amount import DutyAmount
from .email import Email
from .freight_amount import FreightAmount
from .frequency import Frequency
from .invoice_amount import InvoiceAmount
from .invoice_number import InvoiceNumber
from .invoice_type import InvoiceType
from .invoicestatus import Invoicestatus
from .purchase_order import PurchaseOrder
from .shipping_from_zip import ShippingFromZip
from .shippingaddress import Shippingaddress
from .shippingaddressadditional import Shippingaddressadditional
from .shippingcity import Shippingcity
from .shippingcountry import Shippingcountry
from .shippingstate import Shippingstate
from .shippingzip import Shippingzip
from .summary_commodity_code import SummaryCommodityCode
from .tax import Tax
from .terms_conditions import TermsConditions


class BillData(UniversalBaseModel):
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="AdditionalData")
    ] = None
    attachments: typing.Optional[Attachments] = None
    company: typing.Optional[str] = pydantic.Field(default=None)
    """
    Company name of the recipient of the invoice.
    """

    discount: typing.Optional[Discount] = None
    duty_amount: typing_extensions.Annotated[typing.Optional[DutyAmount], FieldMetadata(alias="dutyAmount")] = None
    first_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="firstName")] = pydantic.Field(
        default=None
    )
    """
    First name of the recipient of the invoice.
    """

    freight_amount: typing_extensions.Annotated[
        typing.Optional[FreightAmount], FieldMetadata(alias="freightAmount")
    ] = None
    frequency: typing.Optional[Frequency] = pydantic.Field(default=None)
    """
    Frequency of scheduled invoice.
    """

    invoice_amount: typing_extensions.Annotated[
        typing.Optional[InvoiceAmount], FieldMetadata(alias="invoiceAmount")
    ] = None
    invoice_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="invoiceDate")] = (
        pydantic.Field(default=None)
    )
    """
    Invoice date in any of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    invoice_due_date: typing_extensions.Annotated[
        typing.Optional[Datenullable], FieldMetadata(alias="invoiceDueDate")
    ] = pydantic.Field(default=None)
    """
    Invoice due date in one of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    invoice_end_date: typing_extensions.Annotated[
        typing.Optional[Datenullable], FieldMetadata(alias="invoiceEndDate")
    ] = pydantic.Field(default=None)
    """
    Indicate the date to finish a scheduled invoice cycle (`invoiceType`` = 1) in any of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    invoice_number: typing_extensions.Annotated[
        typing.Optional[InvoiceNumber], FieldMetadata(alias="invoiceNumber")
    ] = pydantic.Field(default=None)
    """
    Invoice number. Identifies the invoice under a paypoint.
    """

    invoice_status: typing_extensions.Annotated[
        typing.Optional[Invoicestatus], FieldMetadata(alias="invoiceStatus")
    ] = None
    invoice_type: typing_extensions.Annotated[typing.Optional[InvoiceType], FieldMetadata(alias="invoiceType")] = None
    items: typing.Optional[typing.List[BillItem]] = pydantic.Field(default=None)
    """
    Array of line items included in the invoice.
    """

    last_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="lastName")] = pydantic.Field(
        default=None
    )
    """
    Last name of the recipient of the invoice.
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes included in the invoice.
    """

    payment_terms: typing_extensions.Annotated[
        typing.Optional[BillDataPaymentTerms], FieldMetadata(alias="paymentTerms")
    ] = None
    purchase_order: typing_extensions.Annotated[
        typing.Optional[PurchaseOrder], FieldMetadata(alias="purchaseOrder")
    ] = None
    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddress], FieldMetadata(alias="shippingAddress1")
    ] = None
    shipping_address_2: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional], FieldMetadata(alias="shippingAddress2")
    ] = None
    shipping_city: typing_extensions.Annotated[typing.Optional[Shippingcity], FieldMetadata(alias="shippingCity")] = (
        None
    )
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry], FieldMetadata(alias="shippingCountry")
    ] = None
    shipping_email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="shippingEmail")] = (
        pydantic.Field(default=None)
    )
    """
    Shipping recipient's contact email address.
    """

    shipping_from_zip: typing_extensions.Annotated[
        typing.Optional[ShippingFromZip], FieldMetadata(alias="shippingFromZip")
    ] = None
    shipping_phone: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="shippingPhone")] = (
        pydantic.Field(default=None)
    )
    """
    Recipient phone number.
    """

    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="shippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[typing.Optional[Shippingzip], FieldMetadata(alias="shippingZip")] = None
    summary_commodity_code: typing_extensions.Annotated[
        typing.Optional[SummaryCommodityCode], FieldMetadata(alias="summaryCommodityCode")
    ] = None
    tax: typing.Optional[Tax] = None
    terms_conditions: typing_extensions.Annotated[
        typing.Optional[TermsConditions], FieldMetadata(alias="termsConditions")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .element import Element
from .label_element import LabelElement
from .page_element import PageElement
from .setting_element import SettingElement


class ReceiptContent(UniversalBaseModel):
    """
    Object containing receipt body configuration
    """

    amount: typing.Optional[Element] = pydantic.Field(default=None)
    """
    Section amount of payment receipt
    """

    contact_us: typing_extensions.Annotated[typing.Optional[Element], FieldMetadata(alias="contactUs")] = (
        pydantic.Field(default=None)
    )
    """
    Section contactUs of payment receipt
    """

    details: typing.Optional[Element] = pydantic.Field(default=None)
    """
    Section payment details of payment receipt
    """

    logo: typing.Optional[Element] = pydantic.Field(default=None)
    """
    Section logo of payment receipt
    """

    message_before_button: typing_extensions.Annotated[
        typing.Optional[LabelElement], FieldMetadata(alias="messageBeforeButton")
    ] = pydantic.Field(default=None)
    """
    Section message of payment receipt
    """

    page: typing.Optional[PageElement] = pydantic.Field(default=None)
    """
    Section page of payment receipt
    """

    payment_button: typing_extensions.Annotated[typing.Optional[LabelElement], FieldMetadata(alias="paymentButton")] = (
        pydantic.Field(default=None)
    )
    """
    Section payment button of payment receipt
    """

    payment_information: typing_extensions.Annotated[
        typing.Optional[Element], FieldMetadata(alias="paymentInformation")
    ] = pydantic.Field(default=None)
    """
    Section payment information of payment receipt
    """

    settings: typing.Optional[SettingElement] = pydantic.Field(default=None)
    """
    The receipt's settings.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

Frequencynotification = typing.Union[
    typing.Literal[
        "one-time", "daily", "weekly", "biweekly", "monthly", "quarterly", "semiannually", "annually", "untilcancelled"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AddMethodResponse,
        AddMethodResponseResponseData,
        ConvertToken,
        CreateAnonymous,
        GetMethodResponse,
        GetMethodResponseResponseData,
        GetMethodResponseResponseDataCustomersItem,
        RequestTokenStorage,
        RequestTokenStoragePaymentMethod,
        Temporary,
        TokenizeAch,
        TokenizeCard,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AddMethodResponse": ".types",
    "AddMethodResponseResponseData": ".types",
    "ConvertToken": ".types",
    "CreateAnonymous": ".types",
    "GetMethodResponse": ".types",
    "GetMethodResponseResponseData": ".types",
    "GetMethodResponseResponseDataCustomersItem": ".types",
    "RequestTokenStorage": ".types",
    "RequestTokenStoragePaymentMethod": ".types",
    "Temporary": ".types",
    "TokenizeAch": ".types",
    "TokenizeCard": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AddMethodResponse",
    "AddMethodResponseResponseData",
    "ConvertToken",
    "CreateAnonymous",
    "GetMethodResponse",
    "GetMethodResponseResponseData",
    "GetMethodResponseResponseDataCustomersItem",
    "RequestTokenStorage",
    "RequestTokenStoragePaymentMethod",
    "Temporary",
    "TokenizeAch",
    "TokenizeCard",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.accounting_field import AccountingField
from ..types.additional_data_string import AdditionalDataString
from ..types.attachments import Attachments
from ..types.bill_query_response import BillQueryResponse
from ..types.billitems import Billitems
from ..types.billstatus import Billstatus
from ..types.comments import Comments
from ..types.datenullable import Datenullable
from ..types.export_format import ExportFormat
from ..types.file_content import FileContent
from ..types.frequency import Frequency
from ..types.idempotency_key import IdempotencyKey
from ..types.payabli_api_response import PayabliApiResponse
from ..types.terms import Terms
from ..types.vendor_data import VendorData
from .types.bill_out_data_scheduled_options import BillOutDataScheduledOptions
from .types.bill_response import BillResponse
from .types.edit_bill_response import EditBillResponse
from .types.get_bill_response import GetBillResponse
from .types.modify_approval_bill_response import ModifyApprovalBillResponse
from .types.set_approved_bill_response import SetApprovedBillResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBillClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_bill(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BillResponse]:
        """
        Creates a bill in an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/single/{jsonable_encoder(entry)}",
            method="POST",
            json={
                "accountingField1": accounting_field_1,
                "accountingField2": accounting_field_2,
                "additionalData": additional_data,
                "attachments": convert_and_respect_annotation_metadata(
                    object_=attachments, annotation=Attachments, direction="write"
                ),
                "billDate": bill_date,
                "billItems": convert_and_respect_annotation_metadata(
                    object_=bill_items, annotation=Billitems, direction="write"
                ),
                "billNumber": bill_number,
                "comments": comments,
                "discount": discount,
                "dueDate": due_date,
                "endDate": end_date,
                "frequency": frequency,
                "lotNumber": lot_number,
                "mode": mode,
                "netAmount": net_amount,
                "scheduledOptions": convert_and_respect_annotation_metadata(
                    object_=scheduled_options, annotation=BillOutDataScheduledOptions, direction="write"
                ),
                "status": status,
                "terms": terms,
                "totalAmount": total_amount,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BillResponse]:
        """
        Delete a file attached to a bill.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a
            request to `/api/Invoice/{idInvoice}`. Here, the filename is
            `0_Bill.pdf`.

            ```json
              "DocumentsRef": {
                "zipfile": "inva_269.zip",
                "filelist": [
                  {
                    "originalName": "Bill.pdf",
                    "zipName": "0_Bill.pdf",
                    "descriptor": null
                  }
                ]
              }
              ```

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/attachedFileFromBill/{jsonable_encoder(id_bill)}/{jsonable_encoder(filename)}",
            method="DELETE",
            params={
                "returnObject": return_object,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_bill(
        self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BillResponse]:
        """
        Deletes a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/{jsonable_encoder(id_bill)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def edit_bill(
        self,
        id_bill: int,
        *,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EditBillResponse]:
        """
        Updates a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EditBillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/{jsonable_encoder(id_bill)}",
            method="PUT",
            json={
                "accountingField1": accounting_field_1,
                "accountingField2": accounting_field_2,
                "additionalData": additional_data,
                "attachments": convert_and_respect_annotation_metadata(
                    object_=attachments, annotation=Attachments, direction="write"
                ),
                "billDate": bill_date,
                "billItems": convert_and_respect_annotation_metadata(
                    object_=bill_items, annotation=Billitems, direction="write"
                ),
                "billNumber": bill_number,
                "comments": comments,
                "discount": discount,
                "dueDate": due_date,
                "endDate": end_date,
                "frequency": frequency,
                "lotNumber": lot_number,
                "mode": mode,
                "netAmount": net_amount,
                "scheduledOptions": convert_and_respect_annotation_metadata(
                    object_=scheduled_options, annotation=BillOutDataScheduledOptions, direction="write"
                ),
                "status": status,
                "terms": terms,
                "totalAmount": total_amount,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditBillResponse,
                    parse_obj_as(
                        type_=EditBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[FileContent]:
        """
        Retrieves a file attached to a bill, either as a binary file or as a Base64-encoded string.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            "DocumentsRef": {
              "zipfile": "inva_269.zip",
              "filelist": [
                {
                  "originalName": "Bill.pdf",
                  "zipName": "0_Bill.pdf",
                  "descriptor": null
                }
              ]
            }

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[FileContent]
            A successful response returns a binary file when `returnObject` is `false`. When `returnObject` is `true`, the response contains the file content as a Base64-encoded string in an object. Due to technical limitations, only the object response is documented here.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/attachedFileFromBill/{jsonable_encoder(id_bill)}/{jsonable_encoder(filename)}",
            method="GET",
            params={
                "returnObject": return_object,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FileContent,
                    parse_obj_as(
                        type_=FileContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_bill(
        self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBillResponse]:
        """
        Retrieves a bill by ID from an entrypoint.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/{jsonable_encoder(id_bill)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBillResponse,
                    parse_obj_as(
                        type_=GetBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_bills(
        self,
        entry: str,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BillQueryResponse]:
        """
        Retrieve a list of bills for an entrypoint. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (`in`, `nin`, `ne`, `eq`)
            - `method` (`in`, `nin`, `eq`, `ne`)
            - `event` (`in`, `nin`, `eq`, `ne`)
            - `target` (`ct`, `nct`, `eq`, `ne`)
            - `status` (`eq`, `ne`)
            - `approvalUserId` (`eq`, `ne`)
            - `parentOrgId` (`ne`, `eq`, `nin`, `in`)
            - `approvalUserEmail` (`eq`, `ne`)
            - `scheduleId` (`ne`, `eq`)

            List of comparison accepted - enclosed between parentheses:
            - `eq` or empty => equal
            - `gt` => greater than
            - `ge` => greater or equal
            - `lt` => less than
            - `le` => less or equal
            - `ne` => not equal
            - `ct` => contains
            - `nct` => not contains
            - `in` => inside array
            - `nin` => not inside array

            List of parameters accepted:
            - `limitRecord` : max number of records for query (default="20", "0" or negative value for all)
            - `fromRecord` : initial record in query
            Example: `totalAmount(gt)=20` returns all records with a `totalAmount` that's greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillQueryResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Query/bills/{jsonable_encoder(entry)}",
            method="GET",
            params={
                "exportFormat": export_format,
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillQueryResponse,
                    parse_obj_as(
                        type_=BillQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_bills_org(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BillQueryResponse]:
        """
        Retrieve a list of bills for an organization. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (in, nin, ne, eq)
            - `method` (in, nin, eq, ne)
            - `event` (in, nin, eq, ne)
            - `target` (ct, nct, eq, ne)
            - `status` (eq, ne)
            - `parentOrgId` (ne, eq, nin, in)
            - `approvalUserId` (eq, ne)
            - `approvalUserEmail` (eq, ne)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillQueryResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Query/bills/org/{jsonable_encoder(org_id)}",
            method="GET",
            params={
                "exportFormat": export_format,
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillQueryResponse,
                    parse_obj_as(
                        type_=BillQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def modify_approval_bill(
        self, id_bill: int, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ModifyApprovalBillResponse]:
        """
        Modify the list of users the bill is sent to for approval.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ModifyApprovalBillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/approval/{jsonable_encoder(id_bill)}",
            method="PUT",
            json=request,
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ModifyApprovalBillResponse,
                    parse_obj_as(
                        type_=ModifyApprovalBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def send_to_approval_bill(
        self,
        id_bill: int,
        *,
        request: typing.Sequence[str],
        autocreate_user: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BillResponse]:
        """
        Send a bill to a user or list of users to approve.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        autocreate_user : typing.Optional[bool]
            Automatically create the target user for approval if they don't exist.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/approval/{jsonable_encoder(id_bill)}",
            method="POST",
            params={
                "autocreateUser": autocreate_user,
            },
            json=request,
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_approved_bill(
        self,
        approved: str,
        id_bill: int,
        *,
        email: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SetApprovedBillResponse]:
        """
        Approve or disapprove a bill by ID.

        Parameters
        ----------
        approved : str
            String representing the approved status. Accepted values: 'true' or 'false'.

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        email : typing.Optional[str]
            Email or username of user modifying approval status.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetApprovedBillResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Bill/approval/{jsonable_encoder(id_bill)}/{jsonable_encoder(approved)}",
            method="GET",
            params={
                "email": email,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetApprovedBillResponse,
                    parse_obj_as(
                        type_=SetApprovedBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBillClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_bill(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BillResponse]:
        """
        Creates a bill in an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/single/{jsonable_encoder(entry)}",
            method="POST",
            json={
                "accountingField1": accounting_field_1,
                "accountingField2": accounting_field_2,
                "additionalData": additional_data,
                "attachments": convert_and_respect_annotation_metadata(
                    object_=attachments, annotation=Attachments, direction="write"
                ),
                "billDate": bill_date,
                "billItems": convert_and_respect_annotation_metadata(
                    object_=bill_items, annotation=Billitems, direction="write"
                ),
                "billNumber": bill_number,
                "comments": comments,
                "discount": discount,
                "dueDate": due_date,
                "endDate": end_date,
                "frequency": frequency,
                "lotNumber": lot_number,
                "mode": mode,
                "netAmount": net_amount,
                "scheduledOptions": convert_and_respect_annotation_metadata(
                    object_=scheduled_options, annotation=BillOutDataScheduledOptions, direction="write"
                ),
                "status": status,
                "terms": terms,
                "totalAmount": total_amount,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BillResponse]:
        """
        Delete a file attached to a bill.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a
            request to `/api/Invoice/{idInvoice}`. Here, the filename is
            `0_Bill.pdf`.

            ```json
              "DocumentsRef": {
                "zipfile": "inva_269.zip",
                "filelist": [
                  {
                    "originalName": "Bill.pdf",
                    "zipName": "0_Bill.pdf",
                    "descriptor": null
                  }
                ]
              }
              ```

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/attachedFileFromBill/{jsonable_encoder(id_bill)}/{jsonable_encoder(filename)}",
            method="DELETE",
            params={
                "returnObject": return_object,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_bill(
        self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BillResponse]:
        """
        Deletes a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/{jsonable_encoder(id_bill)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def edit_bill(
        self,
        id_bill: int,
        *,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EditBillResponse]:
        """
        Updates a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EditBillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/{jsonable_encoder(id_bill)}",
            method="PUT",
            json={
                "accountingField1": accounting_field_1,
                "accountingField2": accounting_field_2,
                "additionalData": additional_data,
                "attachments": convert_and_respect_annotation_metadata(
                    object_=attachments, annotation=Attachments, direction="write"
                ),
                "billDate": bill_date,
                "billItems": convert_and_respect_annotation_metadata(
                    object_=bill_items, annotation=Billitems, direction="write"
                ),
                "billNumber": bill_number,
                "comments": comments,
                "discount": discount,
                "dueDate": due_date,
                "endDate": end_date,
                "frequency": frequency,
                "lotNumber": lot_number,
                "mode": mode,
                "netAmount": net_amount,
                "scheduledOptions": convert_and_respect_annotation_metadata(
                    object_=scheduled_options, annotation=BillOutDataScheduledOptions, direction="write"
                ),
                "status": status,
                "terms": terms,
                "totalAmount": total_amount,
                "vendor": convert_and_respect_annotation_metadata(
                    object_=vendor, annotation=VendorData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EditBillResponse,
                    parse_obj_as(
                        type_=EditBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[FileContent]:
        """
        Retrieves a file attached to a bill, either as a binary file or as a Base64-encoded string.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            "DocumentsRef": {
              "zipfile": "inva_269.zip",
              "filelist": [
                {
                  "originalName": "Bill.pdf",
                  "zipName": "0_Bill.pdf",
                  "descriptor": null
                }
              ]
            }

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[FileContent]
            A successful response returns a binary file when `returnObject` is `false`. When `returnObject` is `true`, the response contains the file content as a Base64-encoded string in an object. Due to technical limitations, only the object response is documented here.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/attachedFileFromBill/{jsonable_encoder(id_bill)}/{jsonable_encoder(filename)}",
            method="GET",
            params={
                "returnObject": return_object,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    FileContent,
                    parse_obj_as(
                        type_=FileContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_bill(
        self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBillResponse]:
        """
        Retrieves a bill by ID from an entrypoint.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/{jsonable_encoder(id_bill)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBillResponse,
                    parse_obj_as(
                        type_=GetBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_bills(
        self,
        entry: str,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BillQueryResponse]:
        """
        Retrieve a list of bills for an entrypoint. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (`in`, `nin`, `ne`, `eq`)
            - `method` (`in`, `nin`, `eq`, `ne`)
            - `event` (`in`, `nin`, `eq`, `ne`)
            - `target` (`ct`, `nct`, `eq`, `ne`)
            - `status` (`eq`, `ne`)
            - `approvalUserId` (`eq`, `ne`)
            - `parentOrgId` (`ne`, `eq`, `nin`, `in`)
            - `approvalUserEmail` (`eq`, `ne`)
            - `scheduleId` (`ne`, `eq`)

            List of comparison accepted - enclosed between parentheses:
            - `eq` or empty => equal
            - `gt` => greater than
            - `ge` => greater or equal
            - `lt` => less than
            - `le` => less or equal
            - `ne` => not equal
            - `ct` => contains
            - `nct` => not contains
            - `in` => inside array
            - `nin` => not inside array

            List of parameters accepted:
            - `limitRecord` : max number of records for query (default="20", "0" or negative value for all)
            - `fromRecord` : initial record in query
            Example: `totalAmount(gt)=20` returns all records with a `totalAmount` that's greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillQueryResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Query/bills/{jsonable_encoder(entry)}",
            method="GET",
            params={
                "exportFormat": export_format,
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillQueryResponse,
                    parse_obj_as(
                        type_=BillQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_bills_org(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BillQueryResponse]:
        """
        Retrieve a list of bills for an organization. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (in, nin, ne, eq)
            - `method` (in, nin, eq, ne)
            - `event` (in, nin, eq, ne)
            - `target` (ct, nct, eq, ne)
            - `status` (eq, ne)
            - `parentOrgId` (ne, eq, nin, in)
            - `approvalUserId` (eq, ne)
            - `approvalUserEmail` (eq, ne)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillQueryResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Query/bills/org/{jsonable_encoder(org_id)}",
            method="GET",
            params={
                "exportFormat": export_format,
                "fromRecord": from_record,
                "limitRecord": limit_record,
                "parameters": parameters,
                "sortBy": sort_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillQueryResponse,
                    parse_obj_as(
                        type_=BillQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def modify_approval_bill(
        self, id_bill: int, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ModifyApprovalBillResponse]:
        """
        Modify the list of users the bill is sent to for approval.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ModifyApprovalBillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/approval/{jsonable_encoder(id_bill)}",
            method="PUT",
            json=request,
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ModifyApprovalBillResponse,
                    parse_obj_as(
                        type_=ModifyApprovalBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def send_to_approval_bill(
        self,
        id_bill: int,
        *,
        request: typing.Sequence[str],
        autocreate_user: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BillResponse]:
        """
        Send a bill to a user or list of users to approve.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        autocreate_user : typing.Optional[bool]
            Automatically create the target user for approval if they don't exist.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/approval/{jsonable_encoder(id_bill)}",
            method="POST",
            params={
                "autocreateUser": autocreate_user,
            },
            json=request,
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BillResponse,
                    parse_obj_as(
                        type_=BillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_approved_bill(
        self,
        approved: str,
        id_bill: int,
        *,
        email: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SetApprovedBillResponse]:
        """
        Approve or disapprove a bill by ID.

        Parameters
        ----------
        approved : str
            String representing the approved status. Accepted values: 'true' or 'false'.

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        email : typing.Optional[str]
            Email or username of user modifying approval status.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetApprovedBillResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Bill/approval/{jsonable_encoder(id_bill)}/{jsonable_encoder(approved)}",
            method="GET",
            params={
                "email": email,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetApprovedBillResponse,
                    parse_obj_as(
                        type_=SetApprovedBillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .item_commodity_code import ItemCommodityCode
from .item_description import ItemDescription
from .item_product_code import ItemProductCode
from .item_product_name import ItemProductName
from .item_unitof_measure import ItemUnitofMeasure


class BillItem(UniversalBaseModel):
    item_categories: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Optional[str]]], FieldMetadata(alias="itemCategories")
    ] = pydantic.Field(default=None)
    """
    Array of tags classifying item or product.
    """

    item_commodity_code: typing_extensions.Annotated[
        typing.Optional[ItemCommodityCode], FieldMetadata(alias="itemCommodityCode")
    ] = None
    item_cost: typing_extensions.Annotated[float, FieldMetadata(alias="itemCost")] = pydantic.Field()
    """
    Item or product price per unit.
    """

    item_description: typing_extensions.Annotated[
        typing.Optional[ItemDescription], FieldMetadata(alias="itemDescription")
    ] = None
    item_mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="itemMode")] = pydantic.Field(
        default=None
    )
    """
    Internal class of item or product: value '0' is only for invoices , '1' for bills and, '2' common for both.
    """

    item_product_code: typing_extensions.Annotated[
        typing.Optional[ItemProductCode], FieldMetadata(alias="itemProductCode")
    ] = None
    item_product_name: typing_extensions.Annotated[
        typing.Optional[ItemProductName], FieldMetadata(alias="itemProductName")
    ] = None
    item_qty: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="itemQty")] = pydantic.Field(
        default=None
    )
    """
    Quantity of item or product.
    """

    item_tax_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemTaxAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Tax amount applied to item or product.
    """

    item_tax_rate: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemTaxRate")] = (
        pydantic.Field(default=None)
    )
    """
    Tax rate applied to item or product.
    """

    item_total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="itemTotalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount in item or product.
    """

    item_unit_of_measure: typing_extensions.Annotated[
        typing.Optional[ItemUnitofMeasure], FieldMetadata(alias="itemUnitOfMeasure")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SplitFundingRefundContent(UniversalBaseModel):
    account_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="accountId")] = pydantic.Field(
        default=None
    )
    """
    The accountId for the account the transaction was routed to.
    """

    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    The amount to refund to this account.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Refund description.
    """

    origination_entry_point: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="originationEntryPoint")
    ] = pydantic.Field(default=None)
    """
    The entrypoint the transaction belongs to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class BillOptions(UniversalBaseModel):
    include_paylink: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="includePaylink")] = (
        pydantic.Field(default=None)
    )
    """
    Flag to indicate if the scheduled invoice includes a payment link.
    """

    include_pdf: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="includePdf")] = pydantic.Field(
        default=None
    )
    """
    Flag to indicate if the scheduled invoice includes a PDF version of invoice
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.entry import Entry
from .raw_client import AsyncRawCheckCaptureClient, RawCheckCaptureClient
from .types.check_capture_response import CheckCaptureResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CheckCaptureClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCheckCaptureClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCheckCaptureClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCheckCaptureClient
        """
        return self._raw_client

    def check_processing(
        self,
        *,
        entry_point: Entry,
        front_image: str,
        rear_image: str,
        check_amount: int,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CheckCaptureResponse:
        """
        Captures a check for Remote Deposit Capture (RDC) using the provided check images and details. This endpoint handles the OCR extraction of check data including MICR, routing number, account number, and amount. See the [RDC guide](/developers/developer-guides/pay-in-rdc) for more details.

        Parameters
        ----------
        entry_point : Entry

        front_image : str
            Base64-encoded front check image. Must be JPEG or PNG format and less than 1MB. Image must show the entire check clearly with no partial, blurry, or illegible portions.

        rear_image : str
            Base64-encoded rear check image. Must be JPEG or PNG format and less than 1MB. Image must show the entire check clearly with no partial, blurry, or illegible portions.

        check_amount : int
            Check amount in cents (maximum 32-bit integer value).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CheckCaptureResponse
            Success response with check processing results.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.check_capture.check_processing(
            entry_point="47abcfea12",
            front_image="/9j/4AAQSkZJRgABAQEASABIAAD...",
            rear_image="/9j/4AAQSkZJRgABAQEASABIAAD...",
            check_amount=12550,
        )
        """
        _response = self._raw_client.check_processing(
            entry_point=entry_point,
            front_image=front_image,
            rear_image=rear_image,
            check_amount=check_amount,
            request_options=request_options,
        )
        return _response.data


class AsyncCheckCaptureClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCheckCaptureClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCheckCaptureClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCheckCaptureClient
        """
        return self._raw_client

    async def check_processing(
        self,
        *,
        entry_point: Entry,
        front_image: str,
        rear_image: str,
        check_amount: int,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CheckCaptureResponse:
        """
        Captures a check for Remote Deposit Capture (RDC) using the provided check images and details. This endpoint handles the OCR extraction of check data including MICR, routing number, account number, and amount. See the [RDC guide](/developers/developer-guides/pay-in-rdc) for more details.

        Parameters
        ----------
        entry_point : Entry

        front_image : str
            Base64-encoded front check image. Must be JPEG or PNG format and less than 1MB. Image must show the entire check clearly with no partial, blurry, or illegible portions.

        rear_image : str
            Base64-encoded rear check image. Must be JPEG or PNG format and less than 1MB. Image must show the entire check clearly with no partial, blurry, or illegible portions.

        check_amount : int
            Check amount in cents (maximum 32-bit integer value).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CheckCaptureResponse
            Success response with check processing results.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.check_capture.check_processing(
                entry_point="47abcfea12",
                front_image="/9j/4AAQSkZJRgABAQEASABIAAD...",
                rear_image="/9j/4AAQSkZJRgABAQEASABIAAD...",
                check_amount=12550,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.check_processing(
            entry_point=entry_point,
            front_image=front_image,
            rear_image=rear_image,
            check_amount=check_amount,
            request_options=request_options,
        )
        return _response.data
